#!/usr/bin/env python

import sys
import os
import sqlite3
from optparse import OptionParser


## Get the version of the running kernel and match it
## against the MRG-RT release database


def open_mrg_database():
	db_path='/var/db/mrg-rt-release/releases.db'
	db = sqlite3.connect(db_path)	
	return db.cursor()

def list_mrg_release(db, release):
	line = """SELECT RtP.Name, P.Version, Rel.Release, Rel.RHRelease
			FROM Packages AS P, Releases AS Rel, RtPackages AS RtP
			WHERE P.NameId = RtP.NameId
				AND P.RelId = Rel.RelId
				AND Rel.Release like '%s%%';""" % (release)
	db.execute(line)
	return db.fetchall()

def find_exact_kernel_errata(db, pkg):
	line = """SELECT Rel.Release, Rel.RHRelease
			FROM Packages AS P, Releases AS Rel, RtPackages AS RtP
			WHERE P.NameId = RtP.NameId
				AND P.RelId = Rel.RelId
				AND RtP.Name = 'kernel-rt'
				AND P.Version = '%s';""" % (pkg)
	db.execute(line)
	return db.fetchall()

def list_kernels(db, pkg):
	if pkg[1]:
		kversion= "%s-%s" % (pkg[0], pkg[1])
	else:
		kversion= "%s" % (pkg[0])
	line = """SELECT P.Version, Rel.Release
			FROM Packages AS P, Releases AS Rel, RtPackages AS RtP
			WHERE P.NameId = RtP.NameId
				AND P.RelId = Rel.RelId
				AND P.Version like '%s%%'
				AND RtP.Name = 'kernel-rt'""" % (kversion)
	db.execute(line)
	return db.fetchall()

def parse_command_line():
	parser = OptionParser('usage: %prog [options]',
		description="Reports to which MRG Errata the running kernel belongs." +
			 "Also lists the packages for a given Errata Release")
	parser.add_option('-s', '--show', action="store_true",
		help='Show MRG Errata based on the running kernel')
	parser.add_option('-u', '--update-etc', action="store_true",
		help='Updates /etc/mrg-rt-release based on the running kernel')
	parser.add_option('-r', '--release', metavar='MRG-RELEASE',
		help='Lists packages for a given MRG RELEASE')
	parser.add_option('-k', '--kernel', metavar='KERNEL-VERSION',
		help='Shows the MRG Errata for a given KERNEL')
	options, args = parser.parse_args()
	if options.show and options.release:
		parser.print_help()
		sys.exit(-3)
	if (not options.release and not options.kernel) \
	   and (not options.update_etc and not options.show):
		parser.print_help()
		sys.exit(0)

	return options, args

def get_next_element(aux): 
	dot = aux.find(".")
	dash = aux.find("-")
	if (dash > 0) and (dash < dot):
		index = dash
		if aux[:2] == "rt":
			index +=2
	else:
		index = dot
	if index < 0:
		return None
	return aux[:index]

def remove_one_element(aux):
	dot = aux.rfind(".")
	dash = aux.rfind("-")
	if dot > dash:
		index = dot
	else:
		index = dash
	if index < 0:
		return None
	return aux[:index]

def get_version_release(pkg):
	idx = pkg.find("-")
	if idx > 0:
		rel = pkg[idx+1:]
		ver = pkg[:idx]
	else:
		rel = None
		ver = pkg
	return (ver, rel)

def prune_element(element):
	if not element:
		return 0
	if element[:2] == "rt":
		element = element[2:]
	if not element.isdigit():
		element = 0
	else:
		element = int(element)
	return element

def compare_element(orig, ver):
	orig = prune_element(orig)
	ver = prune_element(ver)
	return orig - ver

def compare_version_release(vers, pkg):
	auxver, auxrel = get_version_release(vers)
	ver = pkg[0]
	rel = pkg[1]
	if (ver == auxver) and (rel == auxrel):
		return True
	return False

def get_equalized_element(new, count):
	auxver, auxrel = get_version_release(new)
	if count[1]:
		pkg = auxrel.split(".")
		if len(pkg) > count[1]:
			return pkg[count[1]]
		else:
			return "0"
	pkg = auxver.split(".")
	if len(pkg) > count[0]:
		return pkg[count[0]]
	else:
		return "0"

def get_closest_version(vers, pkg, orig):
	res = None
	for aux in vers:
		if compare_version_release(aux[0], pkg):
			return aux
	count = len(pkg[0].split("."))
	size = len (pkg[0]) + 1
	if pkg[1]:
		count = (count, len(pkg[1].split(".")))
		size += len(pkg[1]) + 1
	else:
		count = (count, None)
	org = get_next_element(orig[size:])
	for aux in vers:
		auxver = get_equalized_element(aux[0], count)
		current = compare_element(org, auxver)
		if not res:
			res = aux
			diff = current
			continue
		if abs(current) < abs(diff):
			diff = current
			res = aux
	if diff < 0:
		res = (res[0], "pre-" + res[1])
	else:
		res = (res[0], res[1] + "+")
	return res
	

def select_errata_release(db, pkg, orig):
	vers = list_kernels(db, pkg)
	if not vers:
		return None
	res = get_closest_version(vers, pkg, orig)
	return res

def find_closest_errata(db, pkg):
	orig = pkg
	dbver = None
	while not dbver:
		ver, rel = get_version_release(pkg)
		dbver = select_errata_release(db, (ver, rel), orig)
		pkg = remove_one_element(pkg)
		if not pkg:
			break
	return dbver

def print_errata_pkgs(vers):
	rel = {}
	for aux in vers:
		release = aux[2]
		errata = aux[3]
		name = aux[0] + '-' + aux[1]
		if rel.has_key((errata, release)):
			rel[(errata, release)].append(name)
		else:
			rel[(errata, release)] = [name]
	for k in rel.keys():
		print "MRG-%s - %s" % (k[1], k[0])
		for aux in rel[k]:
			print "\t%s" % (aux)


def send_to_syslog(msg):
	import syslog
	syslog.openlog("mrg-rt-release")
	syslog.syslog(msg)
	syslog.closelog()

def update_etc_mrg_release(msg):
	try:
		etcfile = open("/etc/mrg-rt-release", "w")
		etcfile.write(msg)
		etcfile.close()
	except IOError:
		print "You have no access rights to /etc/mrg-rt-release"
		sys.exit(-1)

#### --------------
if __name__ == '__main__':
	db = open_mrg_database()

	opt, args = parse_command_line()

	if opt.release:
		aux = list_mrg_release(db, opt.release)
		print_errata_pkgs(aux)
		sys.exit(0)

	if opt.kernel:
		pkg = opt.kernel
	else:
		pkg = os.uname()[2].replace('.x86_64', '')

	result = find_exact_kernel_errata(db, pkg)

	if not result:
		release = find_closest_errata(db, pkg)[1]
		errata = 'unsupported kernel'
	else:
		release, errata = result[0]

	db.close()

	msg = "Red Hat MRG Realtime release %s (%s)\n" % (release, errata)
	if opt.update_etc:
		send_to_syslog(msg)
		update_etc_mrg_release(msg)
	else:
		print msg,


 
